#include <avr/io.h>
#include <util/delay.h>

#define output(directions,pin) (directions |= pin) // set port direction for output
#define input(directions,pin) (directions &= (~pin)) // set port direction for input 
#define set(port,pin) (port |= pin) // set port pin
#define clear(port,pin) (port &= (~pin)) // clear port pin
#define pin_test(pins,pin) (pins & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set

#define led_delay() _delay_ms(1) // LED delay

#define led_port PORTA
#define led_E_port PORTB

#define led_direction DDRA
#define led_E_direction DDRB

#define A (1 << PA6)
#define B (1 << PA5)
#define C (1 << PA7)
#define D (1 << PA4)
#define E (1 << PB2)
#define F (1 << PA3)

#define accel_port PORTA
#define ref (0 << REFS1) | (0 << REFS0) // Vcc reference voltage

#define accel_X_min 300
#define accel_X_max 400
#define accel_X (0 << MUX2) | (1 << MUX1) | (0 << MUX0) // PA2

#define accel_Y_min 280
#define accel_Y_max 380
#define accel_Y (0 << MUX2) | (0 << MUX1) | (1 << MUX0) // PA1

// z range: not yet tested
#define accel_Z (0 << MUX2) | (0 << MUX1) | (0 << MUX0) // PA0

int read_ad_pin(uint8_t pin) {
	
	uint8_t hi_value, lo_value;
	int value;
	
	// set the A/D pin
	ADMUX = ref | pin;
	
	// initiate conversion
	ADCSRA |= (1 << ADSC);
	
	// wait for completion
	while (ADCSRA & (1 << ADSC))
		;
	
	// save result
	lo_value = ADCL;
	hi_value = ADCH;
	
	// process result
	value = 256 * hi_value + lo_value;
	
	return value;
	
}

void on_led(uint8_t hi, uint8_t lo) {
	
	if (hi == E) {
		set(led_E_port,hi);
		output(led_E_direction,hi);
	}
	else {
		set(led_port,hi);
		output(led_direction,hi);
	}
	
	if (lo == E) {
		clear(led_E_port,lo);
		output(led_E_direction,lo);
	}
	else {
		clear(led_port,lo);
		output(led_direction,lo);
	}
	
}

void on(uint8_t x, uint8_t y) {
	if (y == 0) {
		if (x == 0)
			on_led(B,A);
		else if (x == 1)
			on_led(C,A);
		else if (x == 2)
			on_led(D,A);
		else if (x == 3)
			on_led(E,A);
		else if (x == 4)
			on_led(F,A);
	}
	else if (y == 1) {
		if (x == 0)
			on_led(A,B);
		else if (x == 1)
			on_led(C,B);
		else if (x == 2)
			on_led(D,B);
		else if (x == 3)
			on_led(E,B);
		else if (x == 4)
			on_led(F,B);
	}
	else if (y == 2) {
		if (x == 0)
			on_led(A,C);
		else if (x == 1)
			on_led(B,C);
		else if (x == 2)
			on_led(D,C);
		else if (x == 3)
			on_led(E,C);
		else if (x == 4)
			on_led(F,C);
	}
	else if (y == 3) {
		if (x == 0)
			on_led(A,D);
		else if (x == 1)
			on_led(B,D);
		else if (x == 2)
			on_led(C,D);
		else if (x == 3)
			on_led(E,D);
		else if (x == 4)
			on_led(F,D);
	}
	else if (y == 4) {
		if (x == 0)
			on_led(A,E);
		else if (x == 1)
			on_led(B,E);
		else if (x == 2)
			on_led(C,E);
		else if (x == 3)
			on_led(D,E);
		else if (x == 4)
			on_led(F,E);
	}
	else if (y == 5) {
		if (x == 0)
			on_led(A,F);
		else if (x == 1)
			on_led(B,F);
		else if (x == 2)
			on_led(C,F);
		else if (x == 3)
			on_led(D,F);
		else if (x == 4)
			on_led(E,F);
	}
}

void off() {
	
	input(led_direction, A);
	input(led_direction, B);
	input(led_direction, C);
	input(led_direction, D);
	input(led_E_direction,E);
	input(led_direction,F);
	
}

int main(void) {
	static int value_X, value_Y;
	static uint8_t actual_X, actual_Y;
	
	// set clock divider to /1
	CLKPR = (1 << CLKPCE);
	CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);
	
	// init A/D
	ADMUX = ref; // Vcc ref
	ADCSRA = (1 << ADEN) // enable
		| (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0); // prescaler /128
	ADCSRB = (0 << ADLAR); // right adjust
	
	// main loop
	while (1) {
				
		value_X = read_ad_pin(accel_X) - accel_X_min;
		value_Y = read_ad_pin(accel_Y) - accel_Y_min;
				
		off();
		
		if (value_X < 20)
			actual_X = 0;
		else if (value_X < 40)
			actual_X = 1;
		else if (value_X < 60)
			actual_X = 2;
		else if (value_X < 80)
			actual_X = 3;
		else
			actual_X = 4;
		
		if (value_Y < 20)
			actual_Y = 0;
		else if (value_Y < 40)
			actual_Y = 1;
		else if (value_Y < 60)
			actual_Y = 2;
		else if (value_Y < 80)
			actual_Y = 3;
		else
			actual_Y = 4;
		
		on(actual_X, actual_Y);
		
		_delay_ms(75);
		
	}
	
}
